<?php

namespace TDevAgency\Erp\Controllers;

use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\View;
use Illuminate\Validation\ValidationException;
use TDevAgency\Cms\Controllers\Controller;
use Illuminate\Http\Request;
use TDevAgency\Erp\Models\Product;
use TDevAgency\Erp\Models\Manufacture;

class ManufactureController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function index()
    {
        $view = View::make('erp::manufactures.index');
        $manufactures = Manufacture::container()->search()->sortable('id')->withCount('products')->paginate(30);
        $view->with('manufactures', $manufactures);
        return $view;
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        $view = View::make('erp::manufactures.single');
        $manufacture = new Manufacture();
        $view->with('manufacture', $manufacture);
        $view->with('formRoute', route('cms.manufactures.store'));
        $view->with('formMethod', 'POST');
        return $view;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return RedirectResponse
     * @throws ValidationException
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'manufacture_name' => 'required|string',
            'manufacture_url' => 'url|nullable',
            'manufacture_image' => 'nullable|image',
        ]);

        $manufacture = new Manufacture();

        $manufacture->manufacture_name = $request->get('manufacture_name') ?? null;
        $manufacture->manufacture_url = $request->get('manufacture_url') ?? null;
        $manufacture->save();
        if ($request->hasFile('manufacture_image')) {
            $manufacture->manufacture_image = App::make('Erp\ImageHelper')->generateImages($request->file('manufacture_image'), $manufacture);
            $manufacture->update();
        }
        return Redirect::route('cms.manufactures.index')->with('toast', [
            'title' => trans('cms::admin.toast.created'),
            'status' => 'success'
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param Manufacture $manufacture
     * @return void
     */
    public function show(Manufacture $manufacture)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param $id
     * @return \Illuminate\Contracts\View\View
     */
    public function edit($id)
    {
        $manufacture = Manufacture::container()->findOrFail($id);
        $view = View::make('erp::manufactures.single');
        $view->with('manufacture', $manufacture);
        $view->with('formRoute', route('cms.manufactures.update', $manufacture->id));
        $view->with('formMethod', 'PUT');
        return $view;
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param $id
     * @return RedirectResponse
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'manufacture_name' => 'required|string',
            'manufacture_url' => 'url|nullable',
            'manufacture_image' => 'nullable|image',
        ]);

        $manufacture = Manufacture::container()->findOrFail($id);

        $manufacture->manufacture_name = $request->get('manufacture_name') ?? null;
        $manufacture->manufacture_url = $request->get('manufacture_url') ?? null;
        $manufacture->update();
        if ($request->hasFile('manufacture_image')) {

            $path = sprintf('%s/%s', $manufacture->getTable(), $manufacture->getId());
            if (Storage::disk('public')->exists($path)) {
                Storage::disk('public')->deleteDirectory($path);
            }

            $manufacture->manufacture_image = App::make('Erp\ImageHelper')->generateImages($request->file('manufacture_image'), $manufacture);
            $manufacture->update();
        }
        return Redirect::route('cms.manufactures.index')->with('toast', [
            'title' => trans('cms::admin.toast.created'),
            'status' => 'success'
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     * @return RedirectResponse
     * @throws Exception
     */
    public function destroy($id)
    {
        $manufacture = Manufacture::container()->findOrFail($id);
        if (Product::where('manufacture_id', $manufacture->id)->count() > 0) {

            return Redirect::back()->with('toast', [
                'title' => trans('cms::admin.toast.not_deleted'),
                'status' => 'danger',
                'body' => trans('erp::erp.products_has_manufacture_error')
            ]);
        }

        $path = sprintf('%s/%s', $manufacture->getTable(), $manufacture->getId());
        if (Storage::disk('public')->exists($path)) {
            Storage::disk('public')->deleteDirectory($path);
        }

        $manufacture->delete();


        return Redirect::back()->with('toast', [
            'title' => trans('cms::admin.toast.deleted'),
            'status' => 'success'
        ]);

    }
}
